-----------------------------------------
-- Addon: Relational DKP
-- By: Shazear 
-- Shizukana <Veterans of the Phoenix>, Malygos
-- e-mail: shazear@hotmail.com
-- Code also used from CTRT
-- References: 
-----------------------------------------


-----------------------------------------------------------
-- Award Functions
-- These are the functions that allocate the DKP to one or many as needed
--
function RDKP_GetAwardPriveledges()
	-- Return true if this is the person who is the RL
	-- Only the RL/Officer In Charge should be doing this.
	-- Get Current Status in the raid
	-- Get Current Rank.
	-- figure that if player CanEditOfficerNote() then they are sufficient rank w/i the guild to be an officer w/ Raid point priv. 
	RDKP_Debug("GetAwardPriveledges");
	-- For right now, only the raid leader can post current Raid Awards.  
	-- This needs to be expanded for Guild-Wide updates.
	if RDKP_b_Debug then
		-- May want to set a switch in the testUI to change this value.
		return true;
	elseif IsRaidLeader() then
		return true;
	end
	return nil;
end

function RDKP_AwardDKPForOther(szName, iDKP)
	-- This is the way for a guildofficer to add DKP to someone outside of a raid.
	RDKP_Debug("RDKP_AwardDKPForOther()")
	if RDKP_IsDKPOfficer() then
		-- Move forward w/ allocating the DKP to each member.
		local szTrans = RDKP_Transaction_Adjustment(szName, iDKP, 0);
		if szTrans then 
			local bResults = RDKP_Guild_SendInfo(szTrans);
			if ( not bResults ) then
				-- Problem 
				RDKP_Debug("DKP Not Awarded for " .. szName);
			end
		end
		RDKP_Table_RaidInfo["RDKP_RaidID"] = nil;
	end
end

function RDKP_AwardGR(szName, iGR)
	-- This is the way for a guildofficer to add GR to someone outside of a raid.
	RDKP_Debug("RDKP_AwardGR()")
	if RDKP_IsDKPOfficer() then
		-- Move forward w/ allocating the DKP to each member.
		local szTrans = RDKP_Transaction_Adjustment(szName, 0, iGR);
		if szTrans then 
			local bResults = RDKP_Guild_SendInfo(szTrans);
			if ( not bResults ) then
				-- Problem 
				RDKP_Debug("GR Not Awarded for " .. szName);
			end
		end
		RDKP_Table_RaidInfo["RDKP_RaidID"] = nil;
	end
end


function RDKP_AwardDKPForKill(szType)
	-- A kill has occured, update all of the current raid members in the zone w/ DKP for this kill.
	RDKP_Debug("RDKP_AwardDKPForKill()")
	if RDKP_GetAwardPriveledges() then
		-- Move forward w/ allocating the DKP to each member.
		for i = 1, GetNumRaidMembers(), 1 do
			-- Get the info for who's in the raid.
			local szName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(i);
			-- Make sure the character was there at the time of the kill:
			RDKP_Debug("raid" .. i .. ": " .. szName .. " in " .. szZone .. " == " .. GetRealZoneText() )
			if ( szZone == GetRealZoneText() ) then
				RDKP_ConditionalDataReset();
				local szTrans = RDKP_Transaction_Raid(szName, RDKP_TABLE_DKP_VALUE[szType], 0);
				if szTrans then 
					local bResults = RDKP_Raid_SendInfo(szTrans);
					if ( not bResults ) then
						-- Problem 
						RDKP_Debug("DKP Not Awarded for " .. szName);
					end
				end
			else
				RDKP_Debug(szName .. " not in Zone");
			end
		end
	end
end

function RDKP_AwardDKPForTime()
	-- Use Chronos for this functionality:
	-- Chronos.scheduleRepeating()
	-- Start repeating either at zone in while in raid, or upon trigger by RL.
	-- Every hour, DKP should be awarded.  
	-- We could set this up so that we award 1 pt every so many minutes based on a 60/DKPperHour value.
	if RDKP_GetAwardPriveledges() then
		-- Move forward w/ allocating the DKP
		for raidIndex = 1, GetNumRaidMembers(), 1 do
			-- Get the info for who's in the raid.
			local szName, rank, subgroup, level, class, fileName, szZone, online, isDead = GetRaidRosterInfo(raidIndex);
			local szTrans = RDKP_Transaction_Raid(szName, 1, 0);
			if szTrans then 
				local bResults = RDKP_Raid_SendInfo(szTrans);
				if ( not bResults ) then
					-- Problem 
					RDKP_Debug("DKP Not Awarded for " .. szName);
				end
			end
		end
	end 
end

function RDKP_AwardGRForLoot(szLoot)
	-- Post kill, someone gets the loot.  
	-- If the loot is at a certain level, then the reciever needs to be credited for it
	-- Also need to track for shards here.
	-- If a shard is created w/ the loot that was just received, then the GR needs to be removed.
	-- Code stolen from CTRT
	if RDKP_GetAwardPriveledges() then
		local sPlayer, sLink;
		local iStart, iEnd, sPlayerName, sItem = string.find(szLoot, "([^%s]+) receives loot: (.+)%.");
		if ( sPlayerName ) then
			sPlayer = sPlayerName;
			sLink = sItem;
		else
			local iStart, iEnd, sItem = string.find(szLoot, "You receive loot: (.+)%.");
			if ( sItem ) then
				sPlayer = UnitName("player");
				sLink = sItem;
			end
		end
		-- Now that we have the gear, we need to get it's value
		if ( sLink and sPlayer ) then
			RDKP_Debug("sLink: " .. sLink)
			RDKP_Debug("sPlayer: " .. sPlayer)
			local iStart, iEnd, sColor, szItem, sName = string.find(sLink, "|c(%x+)|Hitem:(%d+:%d+:%d+:%d+)|h%[(.-)%]|h|r");
			if (sColor and sName) then
				RDKP_Debug("color: " .. sColor .. "\nName: " .. sName)
				if (CT_RaidTracker_RarityTable[sColor] >= 1) then
					local iGR = RDKP_GR_VALUE[sColor][GetRealZoneText()];
					-- Need to check for shards at this point.  If there was a shard created, 
					-- then we need to refund the last transaction.
					if ( RDKP_SearchList(RDKP_Table_DEMats, sName) ) then
						-- This assumes that Blue gear drops Blue Shards/Dust.
						-- The real solution is to get the value of the previous trasaction for this character and return the reciprical amount.
						iGR = iGR * -1;
					elseif ( RDKP_SearchList(RDKP_Table_FreeDrops, sName) ) then
						-- For those items that won't have a cost associated with them... 
						-- Like ZG coins or scarabs, or... ?
						iGR = 0;
					end
					local szTrans = RDKP_Transaction_Raid(sPlayer, 0, iGR);
					RDKP_Raid_SendInfo(szTrans)
				end 
			else
				RDKP_Debug("Looting: sColor and/or sName are nil.")
			end
		end
	end
end


function RDKP_Transaction_Adjustment(szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	-- These adjustments should be unique to themselves.
	local szTransactionKey = RDKP_Build_RaidKeyID();
	RDKP_Debug("szPlayer: " .. szPlayer);
	if (not iDKP) then 
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	if ((iDKP == 0) and (iGR == 0)) then 
		return nil;
	end
	RDKP_Debug("iDKP: " .. iDKP .. "iGR: " ..  iGR);
	return RDKP_Build_Transaction(szTransactionKey, 1, szPlayer, iDKP, iGR);
end

function RDKP_Transaction_Raid(szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	-- These adjustments should be unique to themselves.
	RDKP_Table_RaidInfo.RDKP_TransactionID = RDKP_Table_RaidInfo.RDKP_TransactionID + 1;
	if (not iDKP) then 
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	if ((iDKP == 0) and (iGR == 0)) then 
		return nil;
	end
	return RDKP_Build_Transaction(RDKP_Table_RaidInfo.RDKP_RaidID, RDKP_Table_RaidInfo.RDKP_TransactionID, szPlayer, iDKP, iGR);
end


function RDKP_Build_Transaction(szRaidID, iTransID, szPlayer, iDKP, iGR)
	-- This will be the single atomic operation that will be presented in the raid log.
	-- and communicated to the rest of the guild.
	-- szRaidID = RaidID of the old raid.
	-- iTransID = Transaction ID for that raid.
	-- szPlayer = who gets the amount
	-- iDKP = DKP amount increase
	-- iGR = GR amount increase
	if (szRaidID) then
		local szTransaction = RDKP_SZ_TRANSACTION_PREFIX;
		for k, v in {szRaidID, iTransID, szPlayer, iDKP, iGR} do
			szTransaction = szTransaction .. v .. RDKP_SZ_FIELD_DELIMITER;
		end
		RDKP_Debug("RDKP_Build_Transaction: " .. szTransaction);
		return szTransaction;
	else
		RDKP_Print("Warning: RaidID not avialable.  Please start raid.")
		return nil;
	end
end

function RDKP_Process_Transaction_Raid(szCaptured)
	RDKP_Debug("RDKP_Process_Transaction_Raid(" .. szCaptured .. ")");
	-- For transactions that are occuring during a raid, we were not pulling out the Transaction Prefix from our matching.
	-- Now we do this to ensure that we're just saving the raidID
	szPattern = RDKP_SZ_TRANSACTION_PREFIX .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER
	-- Might want to do some verification of raidIDs at this point, but for now... 
	-- let's just make it work.
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR = string.find(szCaptured,  szPattern );
	if ( not IsRaidLeader() ) then 
		RDKP_Table_RaidInfo["RDKP_TransactionID"] = iTransactionID
	end
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, tonumber(iDKP), tonumber(iGR));
end

function RDKP_Process_Transaction_Historic(szCaptured)
	-- The string passed in already has the transaction prefix pulled out, so we can just manage starting w/ the Raid ID.
	RDKP_Debug("RDKP_Process_Transaction_Historic(" .. szCaptured .. ")");
	szPattern = "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER .. "(.+)" .. RDKP_SZ_FIELD_DELIMITER
	local iStart, iEnd, szRaidID, iTransactionID, szPlayer, iDKP, iGR = string.find(szCaptured,  szPattern );
	return RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, tonumber(iDKP), tonumber(iGR));
end

function RDKP_Process_Transaction(szRaidID, iTransactionID, szPlayer, iDKP, iGR)
	if (not iDKP) then
		iDKP = 0;
	end
	if (not iGR) then
		iGR = 0;
	end
	if (not szPlayer) then
		RDKP_Debug("RDKP_Process_Transaction() where szPlayer == nil... returning");
		return nil;
	end
	RDKP_Debug("RDKP_Process_Transaction(".. szRaidID .. ", " .. iTransactionID .. ", " .. szPlayer .. ", " .. iDKP .. ", " .. iGR .. ")");
	-- Record the transaction.
	if (not RDKP_Table_Raid_Register[szRaidID]) then
		RDKP_Table_Raid_Register[szRaidID] = {};
	end
	RDKP_Table_Raid_Register[szRaidID][iTransactionID] = {
			["player"] = szPlayer, 
			["DKP"] = iDKP,
			["GR"] = iGR
		};
	-- At the time of the Transaction Processing, we should update the summaries.
	RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	return true;
end

function RDKP_UpdateSummarywithTransaction(szPlayer, iDKP, iGR)
	-- Key point here though, is making sure that the master list is already as up-to-date as possible.
	-- Though argueably, if the summary isn't up to date, it will be once I get all the raid-data available onto the local list.
	-- The Player updated here also should be the Main Player, not necessarily the immediate player participating in the raid.
	RDKP_Debug("RDKP_UpdateSummarywithTransaction(".. szPlayer .. ", " .. iDKP .. ", " .. iGR ..")");
	if (not RDKP_Table_CharacterMap[szPlayer]) then 
		-- Populate a character map w/ the existing character
		-- then continue
		RDKP_PlayerMap_AddNotMe(szPlayer);
	end
	local szMainCharacter = RDKP_Table_CharacterMap[szPlayer]["RDKP_SZ_MAINCHAR"];
	if (not RDKP_Table_MasterPlayers[szMainCharacter]) then
		-- Incase the Character is not on the master list, update the master list and try.
		RDKP_UpdateMasterPlayerList();
	end
	RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] = RDKP_Table_MasterPlayers[szMainCharacter]["DKP"] + iDKP;
	RDKP_Table_MasterPlayers[szMainCharacter]["GR"] = RDKP_Table_MasterPlayers[szMainCharacter]["GR"] + iGR;
	RDKP_Table_MasterPlayers[szMainCharacter]["PRI"] =  RDKP_GetPriority(szMainCharacter);
end
